Imports System.IO
Imports System.Configuration.ConfigurationSettings
Module Module1
    Sub Main()
        Dim fs As FileStream
        Dim sr As StreamReader
        Dim sw As StreamWriter
        Dim el As New EventLog
        Dim ex_list(100), net_list(-1) As String
        Dim i, j, completed, alive, LOG_LEVEL As Integer
        Dim got_it As Boolean
        Dim buf, EXCEPTION_LIST, LOG_FILE, LIST_METHOD, DOMAIN, FILTER, OU As String
        Dim p As New Ping

        Console.WriteLine("ShutdownAll")
        Console.WriteLine("Starting: " & Now())

        ' get the configuration
        DOMAIN = System.Configuration.ConfigurationManager.AppSettings("DOMAIN")
        FILTER = System.Configuration.ConfigurationManager.AppSettings("FILTER")
        OU = System.Configuration.ConfigurationManager.AppSettings("OU")
        EXCEPTION_LIST = System.Configuration.ConfigurationManager.AppSettings("EXCEPTION_LIST")
        LOG_FILE = System.Configuration.ConfigurationManager.AppSettings("LOG_FILE")
        LIST_METHOD = System.Configuration.ConfigurationManager.AppSettings("LIST_METHOD")
        LOG_LEVEL = CInt(System.Configuration.ConfigurationManager.AppSettings("LOG_LEVEL"))

        ' get the exception list
        Console.WriteLine("Opening the exception list...")
        If File.Exists(EXCEPTION_LIST) Then
            Try
                fs = New FileStream(EXCEPTION_LIST, FileMode.Open)
                sr = New StreamReader(fs)
            Catch ex As Exception
                Environment.ExitCode = Err.Number
                el.Source = "ShutdownAll_ST"
                el.WriteEntry("Yikes, couldn't open the exception list!", EventLogEntryType.Warning)
                Exit Sub
            End Try

            Do While sr.Peek() >= 0
                buf = sr.ReadLine.ToUpper
                If buf.StartsWith(";") = False Then
                    ex_list(i) = buf
                    i += 1
                End If
                If i > UBound(ex_list) Then
                    ReDim Preserve ex_list(i + 99)
                End If
            Loop
            sr.Close()
            fs.Close()
        Else
            Environment.ExitCode = 1
            el.Source = "ShutdownAll_ST"
            el.WriteEntry("Yikes, couldn't find the exception list!", EventLogEntryType.Warning)
            Exit Sub
        End If

        ' clean up (but leave at least 1 entry)
        If i > 0 Then
            ReDim Preserve ex_list(i - 1)
        Else
            ReDim ex_list(0)
        End If

        ' open the logfile
        Try
            If LOG_LEVEL > 0 Then
                fs = New FileStream(LOG_FILE, FileMode.Append)
                sw = New StreamWriter(fs)
                sw.WriteLine("")
                sw.WriteLine("Starting: " & Now())
            Else
                sw = Nothing
            End If
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            el.Source = "ShutdownAll_ST"
            el.WriteEntry("Yikes, couldn't open the log file!", EventLogEntryType.Warning)
            Exit Sub
        End Try

        ' get the list of PCs on the network
        Console.WriteLine("Getting network list...")
        Select Case LIST_METHOD
            Case "Browse"
                net_list = GetPCNetList(DOMAIN, FILTER)
            Case "NT_Domain"
                net_list = GetPCDomList(DOMAIN, FILTER)
            Case "AD_Domain"
                net_list = GetPCOUList(DOMAIN, OU, FILTER, False)
        End Select

        If UBound(net_list) = -1 Then
            Environment.ExitCode = 1
            el.Source = "ShutdownAll_ST"
            el.WriteEntry("Yikes, couldn't get the network list!", EventLogEntryType.Warning)
            Exit Sub
        End If

        ' set the privileges on this PC
        If Not SetPrivileges("") Then
            Environment.ExitCode = 1
            el.Source = "ShutdownAll_ST"
            el.WriteEntry("Yikes, couldn't set privileges", EventLogEntryType.Warning)
            Exit Sub
        End If

        ' loop thru all PCs 
        Console.WriteLine("Found " & UBound(net_list) + 1 & " computers on the network")
        completed = 0
        buf = ""
        For i = 0 To UBound(net_list)
            ' see if we should exclude this one...
            got_it = False
            For j = 0 To UBound(ex_list)
                If net_list(i) = ex_list(j) Then
                    got_it = True
                    Exit For
                End If
            Next

            ' do the deed!
            If Not got_it Then
                ' Determine whether you can establish a socket connection to remote PC
                Try
                    alive = p.Ping(net_list(i))
                Catch ex As Exception
                    If LOG_LEVEL > 2 Then
                        sw.WriteLine(net_list(i) & "- " & ex.Message)
                    End If
                    Console.Write("?")
                    Continue For
                End Try

                If alive <= 1000 Then
                    If ShutdownPC(net_list(i), True) Then
                        completed += 1
                        If LOG_LEVEL > 2 Then
                            sw.WriteLine(net_list(i) & "- Success!")
                        End If
                        Console.Write(".")
                    Else
                        ErrMsg = ErrMsg.Replace(vbCr, "").Replace(vbLf, "")
                        ' error get recorded in the event log buffer
                        buf &= net_list(i) & "- Error: " & ErrMsg & vbCrLf
                        If LOG_LEVEL > 1 Then
                            sw.WriteLine(net_list(i) & "- Error: " & ErrMsg)
                        End If
                        Console.Write("!")
                    End If
                Else
                    If LOG_LEVEL > 2 Then
                        sw.WriteLine(net_list(i) & "- Didn't answer a ping")
                    End If
                    Console.Write("?")
                End If
            Else
                If LOG_LEVEL > 2 Then
                    sw.WriteLine(net_list(i) & "- Excluded")
                End If
            End If
        Next

        If LOG_LEVEL > 0 Then
            sw.WriteLine("Successfully shutdown " & completed & " (out of " & net_list.Length & ") PCs")
            sw.WriteLine("Ending: " & Now())
        End If
        Console.WriteLine("")
        Console.WriteLine("Successfully shutdown " & completed & " (out of " & net_list.Length & ") PCs")
        If LOG_LEVEL > 0 Then
            sw.Close()
            fs.Close()
        End If
        el.Source = "ShutdownAll_ST"
        el.WriteEntry("Successfully shutdown " & completed & " (out of " & net_list.Length & ") PCs" & vbCrLf & buf, EventLogEntryType.Information)

        Console.WriteLine("Ending: " & Now())
        System.Threading.Thread.Sleep(5000)
    End Sub
End Module
